@@ -32,7 +32,7 @@ import qualified Propellor.Property.Gpg as Gpg
 import qualified Propellor.Property.Systemd as Systemd
 import qualified Propellor.Property.Journald as Journald
 import qualified Propellor.Property.Fail2Ban as Fail2Ban
-import qualified Propellor.Property.Aiccu as Aiccu
+import qualified Propellor.Property.Laptop as Laptop
 import qualified Propellor.Property.OS as OS
 import qualified Propellor.Property.HostingProvider.CloudAtCost as CloudAtCost
 import qualified Propellor.Property.HostingProvider.Linode as Linode
@@ -59,6 +59,7 @@ hosts =                 --                  (o)  `
 	, elephant
 	, beaver
 	, mouse
+	, peregrine
 	, pell
 	, keysafe
 	] ++ monsters
@@ -86,20 +87,24 @@ darkstar :: Host
 darkstar = host "darkstar.kitenet.net" $ props
 	& osDebian Unstable X86_64
 	& ipv6 "2001:4830:1600:187::2"
-	& Aiccu.hasConfig "T18376" "JHZ2-SIXXS"
-
-	& User.nuked (User "nosuchuser") User.YesReallyDeleteHome
+	& Hostname.sane
+	& Apt.serviceInstalledRunning "swapspace"
+	& Laptop.powertopAutoTuneOnBoot
+	& Grub.cmdline_Linux_default "i915.enable_psr=1"
+	! Grub.cmdline_Linux_default "quiet"
 
 	& JoeySites.dkimMilter
 	& JoeySites.postfixSaslPasswordClient
 	-- & JoeySites.alarmClock "*-*-* 7:30" (User "joey")
 	--	"/usr/bin/timeout 45m /home/joey/bin/goodmorning"
+	& JoeySites.laptopSoftware
+	& JoeySites.userDirHtml
 	& Ssh.userKeys (User "joey") hostContext
 		[ (SshRsa, "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC1YoyHxZwG5Eg0yiMTJLSWJ/+dMM6zZkZiR4JJ0iUfP+tT2bm/lxYompbSqBeiCq+PYcSC67mALxp1vfmdOV//LWlbXfotpxtyxbdTcQbHhdz4num9rJQz1tjsOsxTEheX5jKirFNC5OiKhqwIuNydKWDS9qHGqsKcZQ8p+n1g9Lr3nJVGY7eRRXzw/HopTpwmGmAmb9IXY6DC2k91KReRZAlOrk0287LaK3eCe1z0bu7LYzqqS+w99iXZ/Qs0m9OqAPnHZjWQQ0fN4xn5JQpZSJ7sqO38TBAimM+IHPmy2FTNVVn9zGM+vN1O2xr3l796QmaUG1+XLL0shfR/OZbb joey@darkstar")
 		]
-	& imageBuiltFor honeybee
-		(RawDiskImage "/srv/honeybee.img")
-		(Debootstrapped mempty)
+	-- & imageBuiltFor honeybee
+	--	(RawDiskImage "/srv/honeybee.img")
+	--	(Debootstrapped mempty)
 
 gnu :: Host
 gnu = host "gnu.kitenet.net" $ props
@@ -115,7 +120,7 @@ clam :: Host
 clam = host "clam.kitenet.net" $ props
 	& standardSystem Unstable X86_64
 		["Unreliable server. Anything here may be lost at any time!" ]
-	& ipv4 "45.62.211.6"
+	& ipv4 "45.62.211.94"
 
 	& CloudAtCost.decruft
 	& Ssh.hostKeys hostContext
@@ -182,17 +187,12 @@ honeybee = host "honeybee.kitenet.net" $ props
 		[ "Home router and arm git-annex build box." ]
 	
 	& cubietech_Cubietruck
-	& hasPartition
-		( partition EXT2
-			`mountedAt` "/boot"
-			`partLocation` Beginning
-			`setSize` MegaBytes 200
-		)
 	& hasPartition
 		( partition EXT4
 			`mountedAt` "/"
-			`addFreeSpace` MegaBytes 500
+			`setSize` MegaBytes 8000
 		)
+	& JoeySites.cubieTruckOneWire
 	
 	& Apt.installed ["firmware-brcm80211"]
 		-- Workaround for https://bugs.debian.org/844056
@@ -207,6 +207,7 @@ honeybee = host "honeybee.kitenet.net" $ props
 
 	& JoeySites.homePowerMonitor
 		(User "joey")
+		hosts
 		(Context "homepower.joeyh.name")
 		(SshEd25519, "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIMAmVYddg/RgCbIj+cLcEiddeFXaYFnbEJ3uGj9G/EyV joey@honeybee")
 	& JoeySites.homeRouter
@@ -321,6 +322,7 @@ kite = host "kite.kitenet.net" $ props
 	& myDnsPrimary True "kitenet.net"
 		[ (RelDomain "mouse-onion", CNAME $ AbsDomain "htieo6yu2qtcn2j3.onion")
 		, (RelDomain "beaver-onion", CNAME $ AbsDomain "tl4xsvaxryjylgxs.onion")
+		, (RelDomain "peregrine-onion", CNAME $ AbsDomain "ahw47zqw6qszoufl.onion")
 		]
 	& myDnsPrimary True "joeyh.name" []
 	& myDnsPrimary True "ikiwiki.info" []
@@ -425,6 +427,12 @@ mouse = host "mouse.kitenet.net" $ props
 	& Tor.installed
 	& Tor.hiddenServiceAvailable "ssh" (Port 22)
 
+peregrine :: Host
+peregrine = host "peregrine.kitenet.net" $ props
+	& Apt.installed ["ssh"]
+	& Tor.installed
+	& Tor.hiddenServiceAvailable "ssh" (Port 22)
+
 -- Branchable is not completely deployed with propellor yet.
 pell :: Host
 pell = host "pell.branchable.com" $ props
