@@ -1,3 +1,5 @@
+{-# LANGUAGE TypeFamilies #-}
+
 -- | Maintainer: Jelmer Vernooij <jelmer@samba.org>
 
 module Propellor.Property.Aiccu (
@@ -14,10 +16,10 @@ import qualified Propellor.Property.Apt as Apt
 import qualified Propellor.Property.Service as Service
 import qualified Propellor.Property.File as File
 
-installed :: Property NoInfo
+installed :: Property DebianLike
 installed = Apt.installed ["aiccu"]
 
-restarted :: Property NoInfo
+restarted :: Property DebianLike
 restarted = Service.restarted "aiccu"
 
 confPath :: FilePath
@@ -41,12 +43,12 @@ config u t p =
 
 -- | Configures an ipv6 tunnel using sixxs.net, with the given TunneId
 -- and sixx.net UserName.
-hasConfig :: TunnelId -> UserName -> Property HasInfo
-hasConfig t u = prop  `onChange` restarted
+hasConfig :: TunnelId -> UserName -> Property (HasInfo + DebianLike)
+hasConfig t u = prop `onChange` restarted
   where
+  	prop :: Property (HasInfo + UnixLike)
 	prop = withSomePrivData [(Password (u++"/"++t)), (Password u)] (Context "aiccu") $
-		property "aiccu configured" . writeConfig
-	writeConfig :: (((PrivDataField, PrivData) -> Propellor Result) -> Propellor Result) -> Propellor Result
-	writeConfig getpassword = getpassword $ ensureProperty . go
+		property' "aiccu configured" . writeConfig
+	writeConfig getpassword w = getpassword $ ensureProperty w . go
 	go (Password u', p) = confPath `File.hasContentProtected` config u' t p
 	go (f, _) = error $ "Unexpected type of privdata: " ++ show f
