@@ -331,7 +331,9 @@ isInstalled :: Package -> IO Bool
 isInstalled p = isInstalled' [p]
 
 isInstalled' :: [Package] -> IO Bool
-isInstalled' ps = all (== IsInstalled) <$> getInstallStatus ps
+isInstalled' ps = do
+	is <- getInstallStatus ps
+	return $ all (== IsInstalled) is && length is == length ps
 
 data InstallStatus = IsInstalled | NotInstalled
 	deriving (Show, Eq)
@@ -499,12 +501,16 @@ dpkgStatus = "/var/lib/dpkg/status"
 
 -- | Set apt's proxy
 proxy :: Url -> Property (HasInfo + DebianLike)
-proxy u = tightenTargets $
-	proxyInfo `before` proxyConfig `describe` desc
+proxy u = setInfoProperty (proxy' u) (proxyInfo u)
   where
-	proxyInfo = pureInfoProperty desc (InfoVal (HostAptProxy u))
-	proxyConfig = "/etc/apt/apt.conf.d/20proxy" `File.hasContent`
+	proxyInfo = toInfo . InfoVal . HostAptProxy
+
+proxy' :: Url -> Property DebianLike
+proxy' u = tightenTargets $
+	"/etc/apt/apt.conf.d/20proxy" `File.hasContent`
 		[ "Acquire::HTTP::Proxy \"" ++ u ++ "\";" ]
+		`describe` desc
+  where
 	desc = (u ++ " apt proxy selected")
 
 -- | Cause apt to proxy downloads via an apt cacher on localhost
