@@ -541,7 +541,7 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 		, "smtpd_sasl_security_options = noanonymous"
 		, "smtpd_sasl_local_domain = kitenet.net"
 
-		, "# Enable postgrey."
+		, "# Enable postgrey and sasl auth and client certs."
 		, "smtpd_recipient_restrictions = permit_tls_clientcerts,permit_sasl_authenticated,,permit_mynetworks,reject_unauth_destination,check_policy_service inet:127.0.0.1:10023"
 
 		, "# Enable spamass-milter, amavis-milter (opendkim is not enabled because it causes mails forwarded from eg gmail to be rejected)"
@@ -646,7 +646,7 @@ dkimInstalled = go `onChange` Service.restarted "opendkim"
 		& Apt.serviceInstalledRunning "opendkim"
 		& File.dirExists "/etc/mail"
 		& File.hasPrivContent "/etc/mail/dkim.key" (Context "kitenet.net")
-		& File.ownerGroup "/etc/mail/dkim.key" (User "opendkim") (Group "opendkim")
+		& File.ownerGroup "/etc/mail/dkim.key" (User "root") (Group "root")
 		& "/etc/default/opendkim" `File.containsLine`
 			"SOCKET=\"inet:8891@localhost\""
 			`onChange` 
@@ -668,7 +668,6 @@ domainKey = (RelDomain "mail._domainkey", TXT "v=DKIM1; k=rsa; p=MIGfMA0GCSqGSIb
 
 postfixSaslPasswordClient :: Property (HasInfo + DebianLike)
 postfixSaslPasswordClient = combineProperties "postfix uses SASL password to authenticate with smarthost" $ props
-	& Postfix.satellite
 	& Postfix.mappedFile "/etc/postfix/sasl_passwd" 
 		(`File.hasPrivContent` (Context "kitenet.net"))
 	& Postfix.mainCfFile `File.containsLines`
@@ -680,6 +679,9 @@ postfixSaslPasswordClient = combineProperties "postfix uses SASL password to aut
 		, "smtp_sasl_password_maps = hash:/etc/postfix/sasl_passwd"
 		]
 		`onChange` Postfix.reloaded
+	-- Comes after so it does not set relayhost but uses the setting 
+	-- above.
+	& Postfix.satellite
 
 hasPostfixCert :: Context -> Property (HasInfo + UnixLike)
 hasPostfixCert ctx = combineProperties "postfix tls cert installed" $ props
@@ -906,8 +908,8 @@ alarmClock oncalendar (User user) command = combineProperties "goodmorning timer
 		("Login", "LidSwitchIgnoreInhibited", "no")
 
 -- My home power monitor.
-homePowerMonitor :: IsContext c => User -> c -> (SshKeyType, Ssh.PubKeyText) -> Property (HasInfo + DebianLike)
-homePowerMonitor user ctx sshkey = propertyList "home power monitor" $ props
+homePowerMonitor :: IsContext c => User -> [Host] -> c -> (SshKeyType, Ssh.PubKeyText) -> Property (HasInfo + DebianLike)
+homePowerMonitor user hosts ctx sshkey = propertyList "home power monitor" $ props
 	& Apache.installed
 	& Apt.installed ["python", "python-pymodbus", "rrdtool", "rsync"]
 	& File.ownerGroup "/var/www/html" user (userGroup user)
@@ -923,6 +925,7 @@ homePowerMonitor user ctx sshkey = propertyList "home power monitor" $ props
 		`requires` File.ownerGroup (takeDirectory sshkeyfile)
 			user (userGroup user)
 		`requires` File.dirExists (takeDirectory sshkeyfile)
+		`requires` Ssh.knownHost hosts "kitenet.net" user
   where
 	d = "/var/www/html/homepower"
 	sshkeyfile = d </> ".ssh/key"
@@ -947,10 +950,9 @@ homePowerMonitor user ctx sshkey = propertyList "home power monitor" $ props
 		, "[Install]"
 		, "WantedBy=multi-user.target"
 		]
-	-- Only upload when eth0 is up; eg the satellite internet is up.
 	-- Any changes to the rsync command will need my .authorized_keys
 	-- rsync server command to be updated too.
-	rsynccommand = "if ip route | grep '^default' | grep -q eth0; then rsync -e 'ssh -i" ++ sshkeyfile ++ "' -avz rrds/recent/ joey@kitenet.net:/srv/web/homepower.joeyh.name/rrds/recent/; fi"
+	rsynccommand = "rsync -e 'ssh -i" ++ sshkeyfile ++ "' -avz rrds/recent/ joey@kitenet.net:/srv/web/homepower.joeyh.name/rrds/recent/"
 
 -- My home router, running hostapd and dnsmasq for wlan0,
 -- with eth0 connected to a satellite modem, and a fallback ppp connection.
@@ -997,7 +999,7 @@ homeRouter = propertyList "home router" $ props
 	& Apt.installed ["ppp"]
 		`before` File.hasContent "/etc/ppp/peers/provider"
 			[ "user \"joeyh@arczip.com\""
-			, "connect \"/usr/sbin/chat -v -f /etc/chatscripts/pap -T 9734111\""
+			, "connect \"/usr/sbin/chat -v -f /etc/chatscripts/pap -T 3825441\""
 			, "/dev/ttyACM0"
 			, "115200"
 			, "noipdefault"
@@ -1033,3 +1035,81 @@ ipmasq intif = File.hasContent ifupscript
 	ifupscript = "/etc/network/if-up.d/ipmasq"
 	pppupscript = "/etc/ppp/ip-up.d/ipmasq"
 	scriptmode f = f `File.mode` combineModes (readModes ++ executeModes)
+
+laptopSoftware :: Property DebianLike
+laptopSoftware = Apt.installed
+	[ "intel-microcode"
+	, "procmeter3", "xfce4", "procmeter3", "unclutter"
+	, "mplayer", "fbreader", "firefox", "chromium"
+	, "libdatetime-event-sunrise-perl", "libtime-duration-perl"
+	, "network-manager", "gtk-redshift", "powertop"
+	, "gimp", "gthumb", "inkscape", "sozi", "xzgv", "hugin"
+	, "mpc", "mpd", "ncmpc", "sonata", "mpdtoys"
+	, "bsdgames", "nethack-console"
+	, "xmonad", "libghc-xmonad-dev", "libghc-xmonad-contrib-dev"
+	, "ttf-bitstream-vera"
+	, "mairix", "offlineimap", "mutt"
+	, "nmap", "whois", "wireshark", "tcpdump", "iftop"
+	, "udevil", "pmount", "tree", "pv"
+	, "arbtt", "hledger", "bc"
+	, "apache2", "ikiwiki", "libhighlight-perl"
+	, "pal"
+	, "yeahconsole", "xkbset", "xinput"
+	, "assword", "pumpa"
+	, "vorbis-tools", "audacity"
+	, "xul-ext-ublock-origin", "xul-ext-pdf.js", "xul-ext-status4evar"
+	, "vim-syntastic", "vim-fugitive"
+	, "adb", "gthumb"
+	, "w3m", "sm", "weechat"
+	, "borgbackup", "wipe", "smartmontools", "libgfshare-bin"
+	, "units"
+	]
+	`requires` baseSoftware
+	`requires` devSoftware
+
+baseSoftware :: Property DebianLike
+baseSoftware = Apt.installed
+	[ "bash", "bash-completion", "vim", "screen", "less", "moreutils"
+	, "git", "mr", "etckeeper", "git-annex", "ssh", "vim-vimoutliner"
+	]
+
+devSoftware :: Property DebianLike
+devSoftware = Apt.installed
+	[ "build-essential", "debhelper", "devscripts"
+	, "ghc", "cabal-install", "haskell-stack"
+	, "hothasktags", "hdevtools", "hlint"
+	, "gdb", "dpkg-repack", "lintian"
+	, "pristine-tar", "github-backup"
+	]
+
+cubieTruckOneWire :: Property DebianLike
+cubieTruckOneWire = 
+	File.hasContent "/etc/easy-peasy-devicetree-squeezy/my.dts" mydts
+		`onChange` utilitysetup
+		`requires` utilityinstalled
+  where
+	utilityinstalled = Git.cloned (User "root") "https://git.joeyh.name/git/easy-peasy-devicetree-squeezy.git" "/usr/local/easy-peasy-devicetree-squeezy" Nothing
+		`onChange` File.isSymlinkedTo "/usr/local/bin/easy-peasy-devicetree-squeezy" (File.LinkTarget "/usr/local/easy-peasy-devicetree-squeezy/easy-peasy-devicetree-squeezy")
+	utilitysetup = cmdProperty "easy-peasy-devicetree-squeezy"
+		["--debian", "sun7i-a20-cubietruck"]
+		`assume` MadeChange
+	mydts =
+		[ "/* Device tree addition enabling onewire sensors on CubieTruck GPIO pin PG8 */"
+		, "#include <dt-bindings/gpio/gpio.h>"
+		, ""
+		, "/ {"
+		, "\tonewire_device {"
+		, "\t\tcompatible = \"w1-gpio\";"
+		, "\t\tgpios = <&pio 6 8 GPIO_ACTIVE_HIGH>; /* PG8 */"
+		, "\t\tpinctrl-names = \"default\";"
+		, "\t\tpinctrl-0 = <&my_w1_pin>;"
+		, "\t};"
+		, "};"
+		, ""
+		, "&pio {"
+		, "\tmy_w1_pin: my_w1_pin@0 {"
+		, "\t\tallwinner,pins = \"PG8\";"
+		, "\t\tallwinner,function = \"gpio_in\";"
+		, "\t};"
+		, "};"
+		]
