@@ -7,13 +7,16 @@ import Propellor.Types.Info
 
 -- | Boot loader installed on a host.
 data BootloaderInstalled
-	= GrubInstalled
+	= GrubInstalled GrubTarget
 	| FlashKernelInstalled
 	| UbootInstalled (FilePath -> FilePath -> Property Linux)
 	deriving (Typeable)
 
+-- | Platforms that grub can boot.
+data GrubTarget = PC | EFI64 | EFI32 | Coreboot | Xen
+
 instance Show BootloaderInstalled where
-	show GrubInstalled = "GrubInstalled"
+	show (GrubInstalled _) = "GrubInstalled"
 	show FlashKernelInstalled = "FlashKernelInstalled"
 	show (UbootInstalled _) = "UbootInstalled"
 
