@@ -44,8 +44,7 @@ extractInfoEntry (InfoEntry v) = T.cast v
 -- as info, especially type aliases which coud easily lead to bugs.
 -- We want a little bit of dynamic types here, but not too far..
 class (Typeable v, Monoid v, Show v) => IsInfo v where
-	-- | Should info of this type be propagated out of a
-	-- container to its Host?
+	-- | Should this info be propagated out of a container to its Host?
 	propagateInfo :: v -> PropagateInfo
 
 data PropagateInfo
@@ -56,16 +55,15 @@ data PropagateInfo
 
 -- | Any value in the `IsInfo` type class can be added to an Info.
 addInfo :: IsInfo v => Info -> v -> Info
-addInfo (Info l) v = Info (InfoEntry v:l)
+addInfo (Info l) v = Info (l++[InfoEntry v])
 
 -- | Converts any value in the `IsInfo` type class into an Info,
 -- which is otherwise empty.
 toInfo :: IsInfo v => v -> Info
 toInfo = addInfo mempty
 
--- The list is reversed here because addInfo builds it up in reverse order.
 fromInfo :: IsInfo v => Info -> v
-fromInfo (Info l) = mconcat (mapMaybe extractInfoEntry (reverse l))
+fromInfo (Info l) = mconcat (mapMaybe extractInfoEntry l)
 
 -- | Maps a function over all values stored in the Info that are of the
 -- appropriate type.
